@@ -29,16 +29,22 @@ class Error(Exception):
 
 
 class CommError(Error):
-    """Raised for communication errors."""
+    """Raised for communication errors with the PLC.
+
+    For example, loss of socket connection.
+    """
 
 
 class SpecError(Error):
-    """Raised for invalid PLCs, or invalid data from the PLC."""
+    """Raised for invalid PLCs, or invalid data from the PLC.
 
-
-class PlcError(Error):
-    """Raised for errors coming from the PLC."""
+    For example, a PLC defining an indexer that is too small, or using an
+    unsupported version of PILS.
+    """
 
 
 class ApiError(Error):
-    """Raised for invalid API usage."""
+    """Raised for invalid usage of the Zapf API, or the PLC devices.
+
+    For example, trying to write to a read-only device.
+    """
