@@ -29,7 +29,7 @@ from struct import calcsize
 
 from zapf import ApiError, SpecError
 from zapf.spec import FIRST_FLOAT_PARAM, FLOAT32_MAX, FLOAT64_MAX, ParamCMDs, \
-    ParamControl, Parameters, PLCStatus, StatusStruct
+    ParamControl, Parameters, PLCStatus, StatusStruct16, StatusStruct32
 from zapf.util import UncasedMap
 
 START16 = PLCStatus.START << 12
@@ -37,6 +37,17 @@ START32 = PLCStatus.START << 28
 
 
 class Device:
+    """Base class for all specific PILS devices.
+
+    :param number: The device number within the PLC (1-N).
+    :param name: The device name.  This is not currently used by Zapf.
+    :param addr: The base device address.
+    :param typecode: The device's typecode.
+    :param info: The "additional info" dictionary returned by the `.Scanner`.
+    :param io: A `.PlcIO` instance to use for communication with the PLC.
+    :param log: A `logging.Logger` to use for device related logging.
+    """
+
     @classmethod
     def class_for(cls, typecode):
         return TYPECODE_MAP.get(typecode, (None,))[0]
@@ -82,42 +93,55 @@ class Device:
 
         self.log = log
         self.io = io
-        self.init()
+        self._init()
         self.io.register_cache_range(self.addr, self.total_size)
 
     def read_status(self):
-        if self.status_addr is None:
-            if self.target_addr is None:
-                return StatusStruct(PLCStatus.IDLE << 12)
-            if self.read_value() == self.read_target():
-                return StatusStruct(PLCStatus.IDLE << 12)
-            return StatusStruct(PLCStatus.BUSY << 12)
-        if self.status_size == 2:
-            return StatusStruct(self.io.read_u16(self.status_addr))
-        return StatusStruct(self.io.read_u32(self.status_addr) >> 16)
+        """Read the status information of the device.
 
-    def read_extended_status(self):
+        This is a tuple of ``(state, reason, aux, error_id)``.
+
+        ``state`` is the 4-bit state (see `.PLCStatus` for possible values).
+        ``reason`` is the 4-bit reason code (see `.ReasonMap`).  ``aux``
+        contains the up to 24 AUX bits.  ``error_id`` is a device-defined
+        16-bit integer.
+
+        For devices that do not have an error ID field, it is returned as zero.
+        """
         if self.status_addr is None:
-            return (StatusStruct(PLCStatus.IDLE << 12), 0, 0)
+            if self.target_addr and self.read_value() != self.read_target():
+                return (PLCStatus.BUSY, 0, 0, 0)
+            return (PLCStatus.IDLE, 0, 0, 0)
         if self.status_size == 2:
-            return (StatusStruct(self.io.read_u16(self.status_addr)), 0, 0)
+            s = StatusStruct16(self.io.read_u16(self.status_addr))
+            return (s.STATE, s.REASON, s.AUX, 0)
         elif self.status_size == 4:
-            value = self.io.read_u32(self.status_addr)
-            return (StatusStruct(value >> 16), value & 0xFFFF, 0)
+            s = StatusStruct32(self.io.read_u32(self.status_addr))
+            return (s.STATE, s.REASON, s.AUX, 0)
         elif self.status_size == 6:
             value, err_id = self.io.read_fmt(self.status_addr, 'IH')
-            return (StatusStruct(value >> 16), value & 0xFFFF, err_id)
+            s = StatusStruct32(value)
+            return (s.STATE, s.REASON, s.AUX, err_id)
         raise SpecError('invalid status_size')
 
     def change_status(self, initial_states=(), final_state=0):
+        """Change the state of the device.
+
+        Since :ref:`not all state transitions are allowed <pils:state-graph>`,
+        you can list the allowed states in *initial_states*.  If that list is
+        given and the current state is not contained, ``False`` is returned.
+        Else the *final_state* is written and ``True`` is returned.
+
+        If the device does not have a status, ``False`` is always returned.
+        """
         if self.status_addr is None:
             return False
         if initial_states:
-            status = self.read_status()
-            if status.STATE not in initial_states:
+            state = self.read_status()[0]
+            if state not in initial_states:
                 return False
         else:
-            status = StatusStruct()
+            status = StatusStruct16()
         status.STATE = final_state
         if self.status_size == 2:
             self.io.write_u16(self.status_addr, int(status))
@@ -127,39 +151,109 @@ class Device:
 
     # to implement:
 
-    def init(self):
+    def _init(self):
         raise NotImplementedError
 
     def get_limits(self):
+        """Return the limits of the device's data type.
+
+        This is not the same as the limits from the device's meta info
+        (absmin/absmax), but only depends on the value type (int16/float32/...)
+        """
         raise NotImplementedError
 
     def read_value(self):
+        """Read the main value of the device.
+
+        The type of the returned value differs depending on the device type:
+
+        * int for discrete devices
+        * float for analog devices
+        * list of float for vector devices
+        """
         raise NotImplementedError
 
     def read_target(self):
+        """Read the target of the device.
+
+        The returned value is of the same type as `read_value`.
+
+        If the device is read-only and has no target, `.ApiError` is raised.
+        """
         raise ApiError('reading target of a read-only device')
 
     def change_target(self, value):
+        """Change the target of the device.
+
+        This will atomically write the new target (whose type must be the same
+        as returned by `read_value`) and set the state to `START`.
+
+        If the device is read-only and has no target, `.ApiError` is raised.
+        """
         raise ApiError('writing target of a read-only device')
 
     def list_params(self):
+        """Return a list of the names of the device's :ref:`parameters
+        <pils:parameters-functions>`.
+
+        If the device supports no parameters, an empty list is returned.
+        """
         return []
 
     def get_param(self, name):
+        """Read the given parameter.
+
+        The type of the parameter depends on the parameter index, it can be
+        int or float.
+
+        The return value is a tuple of ``(param_cmd, value)`` where
+        ``param_cmd`` represents the reply of the parameter state machine,
+        see `.ParamCMDs`.
+
+        If the device supports no parameters, `.ApiError` is raised.
+        """
         raise ApiError('reading parameter of a device without params')
 
     def set_param(self, name, value):
+        """Set the given parameter.
+
+        The type of the parameter depends on the parameter index, it can be
+        int or float.
+
+        The return value is a tuple of ``(param_cmd, value)`` where
+        ``param_cmd`` represents the reply of the parameter state machine,
+        see `.ParamCMDs`, and ``value`` is the read-back value of the
+        parameter (which might differ from the written value due to clamping,
+        rounding or other correction the PLC makes).
+
+        If the device supports no parameters, `.ApiError` is raised.
+        """
         raise ApiError('writing parameter of a device without params')
 
     def list_funcs(self):
+        """Return a list of the names of the device's `special functions
+        <pils:parameters-functions>`.
+
+        If the device supports no special functions, an empty list is returned.
+        """
         return []
 
     def exec_func(self, name, value=None):
+        """Execute the given special function with an argument.
+
+        The return value is a tuple of ``(param_cmd, value)`` where
+        ``param_cmd`` represents the reply of the parameter state machine,
+        see `.ParamCMDs`, and ``value`` is the return value of the function.
+
+        If the device supports no special functions, `.ApiError` is raised.
+        """
         raise ApiError('executing function of a device without functions')
 
 
 class DiscreteDevice(Device):
-    def init(self):
+    """Base class used for discrete devices."""
+
+    def _init(self):
         pass
 
     def get_limits(self):
@@ -193,23 +287,32 @@ class DiscreteDevice(Device):
 
 
 class SimpleDiscreteIn(DiscreteDevice):
-    pass
+    """Class for :ref:`pils:dev-simplediscreteinput`,
+    :ref:`pils:dev-simplediscrete32input`,
+    :ref:`pils:dev-simplediscrete64input`."""
 
 
 class SimpleDiscreteOut(DiscreteDevice):
-    pass
+    """Class for :ref:`pils:dev-simplediscreteoutput`,
+    :ref:`pils:dev-simplediscrete32output`,
+    :ref:`pils:dev-simplediscrete64output`."""
 
 
 class DiscreteIn(DiscreteDevice):
-    pass
+    """Class for :ref:`pils:dev-discreteinput`,
+    :ref:`pils:dev-discrete32input`, :ref:`pils:dev-discrete64input`."""
 
 
 class DiscreteOut(DiscreteDevice):
-    pass
+    """Class for :ref:`pils:dev-discreteoutput`,
+    :ref:`pils:dev-discrete32output`, :ref:`pils:dev-discrete64output`."""
 
 
 class Keyword(DiscreteDevice):
-    def init(self):
+    """Class for :ref:`pils:dev-keyword`, :ref:`pils:dev-keyword32`,
+    :ref:`pils:dev-keyword64`."""
+
+    def _init(self):
         self.target_addr = self.addr
 
     def get_limits(self):
@@ -223,13 +326,17 @@ class Keyword(DiscreteDevice):
 
 
 class StatusWord(Keyword):
-    def init(self):
-        Keyword.init(self)
+    """Class for :ref:`pils:dev-statusword`, :ref:`pils:dev-extstatusword`."""
+
+    def _init(self):
+        Keyword._init(self)
         self.status_addr = self.addr
 
 
 class AnalogDevice(Device):
-    def init(self):
+    """Base class for analog valued devices (except vector devices)."""
+
+    def _init(self):
         pass
 
     def get_limits(self):
@@ -273,28 +380,33 @@ class AnalogDevice(Device):
 
 
 class SimpleAnalogIn(AnalogDevice):
-    pass
+    """Class for :ref:`pils:dev-simpleanaloginput`,
+    :ref:`pils:dev-simpleanalog64input`."""
 
 
 class SimpleAnalogOut(AnalogDevice):
-    pass
+    """Class for :ref:`pils:dev-simpleanalogoutput`,
+    :ref:`pils:dev-simpleanalog64output`."""
 
 
 class AnalogIn(AnalogDevice):
-    pass
+    """Class for :ref:`pils:dev-analoginput`, :ref:`pils:dev-analog64input`."""
 
 
 class AnalogOut(AnalogDevice):
-    pass
+    """Class for :ref:`pils:dev-analogoutput`,
+    :ref:`pils:dev-analog64output`."""
 
 
 class RealValue(AnalogDevice):
-    def init(self):
+    """Class for :ref:`pils:dev-realvalue`, :ref:`pils:dev-realvalue64`."""
+
+    def _init(self):
         self.target_addr = self.addr
 
 
 class FlatParams:
-    def init(self):
+    def _init(self):
         if self.num_params != len(self.params):
             raise SpecError('mismatch between parameter count between '
                             'typecode and parameter indices from indexer '
@@ -345,17 +457,17 @@ class FlatParams:
 
 
 class FlatIn(FlatParams, AnalogDevice):
-    pass
+    """Class for :ref:`pils:dev-flatinput`, :ref:`pils:dev-flat64input`."""
 
 
 class FlatOut(FlatParams, AnalogDevice):
-    pass
+    """Class for :ref:`pils:dev-flatoutput`, :ref:`pils:dev-flat64output`."""
 
 
 class ParamInterface:
     param_timeout = 1
 
-    def init(self):
+    def _init(self):
         self.param_map = UncasedMap(*((p, Parameters[p]) for p in self.params))
         self.func_map = UncasedMap(*((p, Parameters[p]) for p in self.funcs))
         self.param_sm = ParamControl()
@@ -476,15 +588,17 @@ class ParamInterface:
 
 
 class ParamIn(ParamInterface, AnalogDevice):
-    pass
+    """Class for :ref:`pils:dev-paraminput`, :ref:`pils:dev-param64input`."""
 
 
 class ParamOut(ParamInterface, AnalogDevice):
-    pass
+    """Class for :ref:`pils:dev-paramoutput`, :ref:`pils:dev-param64output`."""
 
 
 class VectorDevice(Device):
-    def init(self):
+    """Base class for Vector devices."""
+
+    def _init(self):
         pass
 
     def get_limits(self):
@@ -503,10 +617,13 @@ class VectorDevice(Device):
 
 
 class VectorIn(ParamInterface, VectorDevice):
-    pass
+    """Class for :ref:`pils:dev-vectorinput`, :ref:`pils:dev-vector64input`."""
 
 
 class VectorOut(ParamInterface, VectorDevice):
+    """Class for :ref:`pils:dev-vectoroutput`,
+    :ref:`pils:dev-vector64output`."""
+
     def read_target(self):
         if self.value_size == 4:
             return self.io.read_f32s(self.target_addr, self.num_values)
