@@ -33,8 +33,8 @@ class Indexer:
         self.io = io
         self.log = log
         self.magicstr = None
-        self.addr = None
-        self.size = None
+        self.indexer_addr = None
+        self.indexer_size = None
         self.num_devices = 0
         self.plc_name = None
         self.plc_version = None
@@ -46,7 +46,7 @@ class Indexer:
         Afterwards, checks the indexer and reads extended meta information
         about the PLC if wanted.
         """
-        if self.addr:
+        if self.indexer_addr and self.indexer_size and self.plc_author:
             return
 
         # check MAGIC
@@ -60,13 +60,13 @@ class Indexer:
         addr = self.io.read_u16(spec.OFFSET_ADDR)
         if addr < 6 or addr & 1:
             raise SpecError('indexer offset %s is invalid' % addr)
-        self.addr = addr
+        self.indexer_addr = addr
 
         # query indexer size
         size = self.query_word(spec.INDEXER_DEV, spec.INFO_SIZE)
         if size < 22 or size > 66 or size & 1:
             raise SpecError('indexer size %s is invalid' % size)
-        self.size = size
+        self.indexer_size = size
 
         # query indexer size and offset again, through the info struct,
         # and ensure consistency
@@ -94,7 +94,7 @@ class Indexer:
         # fields in the infostruct: typecode, size, addr, unitcode, unitexp,
         # flags, absmin, absmax, the rest is the name
         result = self.query_data(devnum, spec.INFO_STRUCT,
-                                 'HHHBbIII%ds' % (self.size - 22))
+                                 'HHHBbIII%ds' % (self.indexer_size - 22))
         # convert min/max; these are floats, but we need to potentially word-
         # swap them
         absmin = self.io.float_from_dword(result[6])
@@ -124,20 +124,21 @@ class Indexer:
                 if byte & (1 << bit)]
 
     def query_bytes(self, devnum, infotype):
-        return self.query_data(devnum, infotype, '%ds' % (self.size - 2))[0]
+        return self.query_data(devnum, infotype,
+                               '%ds' % (self.indexer_size - 2))[0]
 
     # even lower level methods to do an indexer transaction
 
     def query_data(self, devnum, infotype, fmt):
         request = infotype << 8 | devnum
-        self.io.write_u16s(self.addr, [request])
+        self.io.write_u16s(self.indexer_addr, [request])
         for i in range(32):
-            reply = self.io.read_fmt(self.addr, 'H' + fmt)
+            reply = self.io.read_fmt(self.indexer_addr, 'H' + fmt)
             if reply[0] == request | 0x8000:
                 return reply[1:]
             # refresh the query if it got overwritten by a different host
             if reply[0] & 0x7fff != request:
-                self.io.write_u16s(self.addr, [request])
+                self.io.write_u16s(self.indexer_addr, [request])
             time.sleep(0.001*i*i)
         raise SpecError('indexer not responding in time!')
 
