@@ -30,7 +30,7 @@ from struct import Struct, pack, unpack, unpack_from
 from zapf import ApiError, CommError
 from zapf.proto import Protocol
 
-MB_ADDR_RE = re.compile(r'modbus://(.+?)(:(\d+))?/(\d+)$')
+MB_ADDR_RE = re.compile(r'modbus://(.+?)(:(\d+))?(?:/(\d+)?)?$')
 
 DEFAULT_PORT = 502
 
@@ -55,7 +55,7 @@ class ModbusProtocol(Protocol):
             port = int(adr.group(3))
         else:
             port = DEFAULT_PORT
-        self._slave = int(adr.group(4))
+        self._slave = int(adr.group(4) or 0)
         self._iphostport = (host, port)
         self._socket = None
         self._trans_id = 1
